Module: Kameleoon::Events::EventHandler

Included in:
DataFileUpdateHandler, HttpRequestHandler
Defined in:
lib/kameleoon/events/event_handler.rb

Overview

Base module for all SDK event handlers. See EventType for the supported event types and their corresponding handler modules.

Including the handler modules is optional: any object which responds to the methods required by the selected event type is accepted by KameleoonClient#set_event_handler.