Class: Kameleoon::Events::DataFileUpdateEvent
- Inherits:
-
Object
- Object
- Kameleoon::Events::DataFileUpdateEvent
- Defined in:
- lib/kameleoon/events/data_file_update_event.rb
Overview
Describes an update of the SDK data file (configuration) reported to DataFileUpdateHandler.
Defined Under Namespace
Modules: Source
Instance Attribute Summary collapse
-
#date_modified ⇒ Integer
readonly
The date of the last modification of the data file, in Unix time milliseconds.
-
#source ⇒ Symbol
readonly
The source of the data file update, one of the
Sourceconstants.
Instance Method Summary collapse
-
#initialize(source, date_modified) ⇒ DataFileUpdateEvent
constructor
A new instance of DataFileUpdateEvent.
- #to_s ⇒ Object
Constructor Details
#initialize(source, date_modified) ⇒ DataFileUpdateEvent
Returns a new instance of DataFileUpdateEvent.
27 28 29 30 |
# File 'lib/kameleoon/events/data_file_update_event.rb', line 27 def initialize(source, date_modified) @source = source @date_modified = date_modified end |
Instance Attribute Details
#date_modified ⇒ Integer (readonly)
Returns The date of the last modification of the data file, in Unix time milliseconds.
25 26 27 |
# File 'lib/kameleoon/events/data_file_update_event.rb', line 25 def date_modified @date_modified end |
#source ⇒ Symbol (readonly)
Returns The source of the data file update, one of the Source constants.
21 22 23 |
# File 'lib/kameleoon/events/data_file_update_event.rb', line 21 def source @source end |
Instance Method Details
#to_s ⇒ Object
32 33 34 |
# File 'lib/kameleoon/events/data_file_update_event.rb', line 32 def to_s "DataFileUpdateEvent{source:#{@source},date_modified:#{@date_modified}}" end |